==> Pcore-v0.45.3.tar.gz: Pcore-v0.45.3/lib/Pcore/API/BitBucket.pm <==
package Pcore::API::BitBucket;

use Pcore -class, -result;
use Pcore::Util::Scalar qw[is_plain_coderef];
use Pcore::API::SCM::Const qw[:SCM_TYPE];

has username => ( is => 'ro', isa => Str, required => 1 );
has password => ( is => 'ro', isa => Str, required => 1 );

has _auth => ( is => 'lazy', isa => Str, init_arg => undef );

sub BUILDARGS ( $self, $args = undef ) {
    $args->{username} ||= $ENV->user_cfg->{BITBUCKET}->{username} if $ENV->user_cfg->{BITBUCKET}->{username};

    $args->{password} ||= $ENV->user_cfg->{BITBUCKET}->{password} if $ENV->user_cfg->{BITBUCKET}->{password};

    return $args;
}

sub _build__auth ($self) {
    return 'Basic ' . P->data->to_b64( "$self->{username}:$self->{password}", q[] );
}

sub _req1 ( $self, $method, $endpoint, $data, $cb ) {
    my $blocking_cv = defined wantarray ? AE::cv : undef;

    P->http->$method(
        'https://bitbucket.org/api/1.0' . $endpoint,
        headers => {
            AUTHORIZATION => $self->_auth,
            CONTENT_TYPE  => 'application/x-www-form-urlencoded; charset=UTF-8',
        },
        body => $data ? P->data->to_uri($data) : undef,
        on_finish => sub ($res) {
            my $api_res;

            if ( !$res ) {
                $api_res = result [ $res->status, $res->reason ], $res->body;
            }
            else {
                my $data = $res->body && $res->body->$* ? P->data->from_json( $res->body ) : undef;

                $api_res = result $res->status, $data;
            }

            $cb->($api_res) if $cb;

            $blocking_cv->send($api_res) if $blocking_cv;

            return;
        }
    );

    return $blocking_cv ? $blocking_cv->recv : ();
}

sub _req2 ( $self, $method, $endpoint, $data, $cb ) {
    my $blocking_cv = defined wantarray ? AE::cv : undef;

    P->http->$method(
        'https://api.bitbucket.org/2.0' . $endpoint,
        headers => {
            AUTHORIZATION => $self->_auth,
            CONTENT_TYPE  => 'application/json',
        },
        body => $data ? P->data->to_json($data) : undef,
        on_finish => sub ($res) {
            my $data = $res->body && $res->body->$* ? P->data->from_json( $res->body ) : undef;

            my $api_res;

            if ( !$res ) {
                $api_res = result [ $res->status, $data->{error}->{message} // $res->reason ];
            }
            else {
                $api_res = result $res->status, $data;
            }

            $cb->($api_res) if $cb;

            $blocking_cv->send($api_res) if $blocking_cv;

            return;
        }
    );

    return $blocking_cv ? $blocking_cv->recv : ();
}

# https://developer.atlassian.com/bitbucket/api/2/reference/resource/repositories/%7Busername%7D/%7Brepo_slug%7D#post
sub create_repo ( $self, $repo_id, @args ) {
    my $blocking_cv = defined wantarray ? AE::cv : undef;

    my $cb = is_plain_coderef $args[-1] ? pop @args : undef;

    my %args = (

        # common atts
        description => undef,
        has_issues  => 1,
        has_wiki    => 1,
        is_private  => 0,

        # bitbucket attrs
        scm         => $SCM_TYPE_HG,
        fork_police => 'allow_forks',    # allow_forks, no_public_forks, no_forks
        language    => 'perl',
        @args
    );

    return $self->_req2( 'post', "/repositories/$repo_id", \%args, $cb );
}

# https://developer.atlassian.com/bitbucket/api/2/reference/resource/repositories/%7Busername%7D/%7Brepo_slug%7D#delete
sub delete_repo ( $self, $repo_id, $cb = undef ) {
    return $self->_req2( 'delete', "/repositories/$repo_id", undef, $cb );
}

# VERSIONS
# https://developer.atlassian.com/bitbucket/api/2/reference/resource/repositories/%7Busername%7D/%7Brepo_slug%7D/versions
sub get_versions ( $self, $repo_id, $cb = undef ) {
    my $blocking_cv = defined wantarray ? AE::cv : undef;

    my $versions;

    my $get = sub ($page) {
        my $sub = __SUB__;

        $self->_req2(
            'get',
            "/repositories/$repo_id/versions?page=$page&pagelen=100",
            undef,
            sub ($res) {
                if ($res) {
                    for my $ver ( $res->{data}->{values}->@* ) {
                        $versions->{ $ver->{name} } = $ver->{links}->{self}->{href};
                    }

                    if ( $res->{data}->{next} ) {
                        $sub->( ++$page );
                    }
                    else {
                        my $api_res = result 200, $versions;

                        $cb->($api_res) if $cb;

                        $blocking_cv->($api_res) if $blocking_cv;
                    }
                }
                else {
                    $cb->($res) if $cb;

                    $blocking_cv->($res) if $blocking_cv;
                }

                return;
            }
        );

        return;
    };

    $get->(1);

    return $blocking_cv ? $blocking_cv->recv : ();
}

# https://confluence.atlassian.com/bitbucket/issues-resource-296095191.html#issuesResource-POSTanewversion
sub create_version ( $self, $repo_id, $ver, $cb = undef ) {
    return $self->_req1(
        'post',
        "/repositories/$repo_id/issues/versions",
        { name => version->parse($ver)->normal },
        sub ($res) {
            if ( !$res && $res->{data}->$* =~ /already exists/sm ) {
                $res->set_status(200);
            }

            $cb->($res) if $cb;

            return;
        }
    );
}

# MILESTONES
# https://developer.atlassian.com/bitbucket/api/2/reference/resource/repositories/%7Busername%7D/%7Brepo_slug%7D/milestones
sub get_milestones ( $self, $repo_id, $cb = undef ) {
    my $blocking_cv = defined wantarray ? AE::cv : undef;

    my $versions;

    my $get = sub ($page) {
        my $sub = __SUB__;

        $self->_req2(
            'get',
            "/repositories/$repo_id/milestones?page=$page&pagelen=100",
            undef,
            sub ($res) {
                if ($res) {
                    for my $ver ( $res->{data}->{values}->@* ) {
                        $versions->{ $ver->{name} } = $ver->{links}->{self}->{href};
                    }

                    if ( $res->{data}->{next} ) {
                        $sub->( ++$page );
                    }
                    else {
                        my $api_res = result 200, $versions;

                        $cb->($api_res) if $cb;

                        $blocking_cv->($api_res) if $blocking_cv;
                    }
                }
                else {
                    $cb->($res) if $cb;

                    $blocking_cv->($res) if $blocking_cv;
                }

                return;
            }
        );

        return;
    };

    $get->(1);

    return $blocking_cv ? $blocking_cv->recv : ();
}

# https://confluence.atlassian.com/bitbucket/issues-resource-296095191.html#issuesResource-POSTanewmilestone
sub create_milestone ( $self, $repo_id, $ver, $cb = undef ) {
    return $self->_req1(
        'post',
        "/repositories/$repo_id/issues/milestones",
        { name => version->parse($ver)->normal },
        sub ($res) {
            if ( !$res && $res->{data}->$* =~ /already exists/sm ) {
                $res->set_status(200);
            }

            $cb->($res) if $cb;

            return;
        }
    );
}

# TODO ISSUES
# https://confluence.atlassian.com/bitbucket/issues-resource-296095191.html#issuesResource-GETalistofissuesinarepository%27stracker
sub get_issues ( $self, @ ) {
    my $cb = $_[-1];

    my %args = (
        limit     => 50,
        sort      => 'priority',    # priority, kind, version, component, milestone
        status    => undef,
        milestone => undef,
        @_[ 1 .. $#_ - 1 ],
    );

    P->http->get(                   #
        "https://bitbucket.org/api/1.0/repositories/@{[$self->id]}/issues/?" . P->data->to_uri( \%args ),
        headers   => { AUTHORIZATION => $self->auth },
        on_finish => sub ($res) {
            if ( !$res ) {
                my $data = eval { P->data->from_json( $res->body ) };

                $cb->( result [ $res->status, $data->{error}->{message} || $res->reason ] );
            }
            else {
                my $data = eval { P->data->from_json( $res->body ) };

                if ($@) {
                    $cb->( result [ 500, 'Error decoding respnse' ] );
                }
                else {
                    my $issues;

                    for my $issue ( $data->{issues}->@* ) {
                        $issue->{api} = $self;

                        push $issues->@*, bless $issue, 'Pcore::API::Bitbucket::Issue';
                    }

                    $cb->( result 200, $issues );
                }
            }

            return;
        },
    );

    return;
}

# https://confluence.atlassian.com/bitbucket/issues-resource-296095191.html#issuesResource-GETanindividualissue
sub get_issue ( $self, $id, $cb ) {
    P->http->get(
        "https://bitbucket.org/api/1.0/repositories/@{[$self->id]}/issues/$id",
        headers   => { AUTHORIZATION => $self->auth },
        on_finish => sub ($res) {
            if ( !$res ) {
                my $data = eval { P->data->from_json( $res->body ) };

                $cb->( result [ $res->status, $data->{error}->{message} || $res->reason ] );
            }
            else {
                my $data = eval { P->data->from_json( $res->body ) };

                if ($@) {
                    $cb->( result [ 500, 'Error decoding respnse' ] );
                }
                else {
                    $data->{api} = $self;

                    $cb->( result 200, bless $data, 'Pcore::API::Bitbucket::Issue' );
                }
            }

            return;
        },
    );

    return;
}

sub set_issue_status ( $self, $id, $status, $cb ) {
    my $issue = Pcore::API::Bitbucket::Issue->new( { api => $self } );

    $issue->{local_id} = $id;

    $issue->set_status( $status, $cb );

    return;
}

1;
## -----SOURCE FILTER LOG BEGIN-----
##
## PerlCritic profile "pcore-script" policy violations:
## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
## | Sev. | Lines                | Policy                                                                                                         |
## |======+======================+================================================================================================================|
## |    1 | 96                   | CodeLayout::RequireTrailingCommas - List declaration without trailing comma                                    |
## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
##
## -----SOURCE FILTER LOG END-----
__END__
=pod

=encoding utf8

=head1 NAME

Pcore::API::BitBucket

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 ATTRIBUTES

=head1 METHODS

=head1 SEE ALSO

=cut

==> Pcore-v0.48.2.tar.gz: Pcore-v0.48.2/lib/Pcore/API/BitBucket.pm <==
package Pcore::API::BitBucket;

use Pcore -class, -result;
use Pcore::Util::Scalar qw[is_plain_coderef];
use Pcore::API::SCM::Const qw[:SCM_TYPE];

has username => ( is => 'ro', isa => Str, required => 1 );
has password => ( is => 'ro', isa => Str, required => 1 );

has _auth => ( is => 'lazy', isa => Str, init_arg => undef );

sub BUILDARGS ( $self, $args = undef ) {
    $args->{username} ||= $ENV->user_cfg->{BITBUCKET}->{username} if $ENV->user_cfg->{BITBUCKET}->{username};

    $args->{password} ||= $ENV->user_cfg->{BITBUCKET}->{password} if $ENV->user_cfg->{BITBUCKET}->{password};

    return $args;
}

sub _build__auth ($self) {
    return 'Basic ' . P->data->to_b64( "$self->{username}:$self->{password}", q[] );
}

sub _req1 ( $self, $method, $endpoint, $data, $cb ) {
    my $blocking_cv = defined wantarray ? AE::cv : undef;

    P->http->$method(
        'https://bitbucket.org/api/1.0' . $endpoint,
        headers => {
            AUTHORIZATION => $self->_auth,
            CONTENT_TYPE  => 'application/x-www-form-urlencoded; charset=UTF-8',
        },
        body => $data ? P->data->to_uri($data) : undef,
        on_finish => sub ($res) {
            my $api_res;

            if ( !$res ) {
                $api_res = result [ $res->status, $res->reason ], $res->body;
            }
            else {
                my $data = $res->body && $res->body->$* ? P->data->from_json( $res->body ) : undef;

                $api_res = result $res->status, $data;
            }

            $cb->($api_res) if $cb;

            $blocking_cv->send($api_res) if $blocking_cv;

            return;
        }
    );

    return $blocking_cv ? $blocking_cv->recv : ();
}

sub _req2 ( $self, $method, $endpoint, $data, $cb ) {
    my $blocking_cv = defined wantarray ? AE::cv : undef;

    P->http->$method(
        'https://api.bitbucket.org/2.0' . $endpoint,
        headers => {
            AUTHORIZATION => $self->_auth,
            CONTENT_TYPE  => 'application/json',
        },
        body => $data ? P->data->to_json($data) : undef,
        on_finish => sub ($res) {
            my $data = $res->body && $res->body->$* ? P->data->from_json( $res->body ) : undef;

            my $api_res;

            if ( !$res ) {
                $api_res = result [ $res->status, $data->{error}->{message} // $res->reason ];
            }
            else {
                $api_res = result $res->status, $data;
            }

            $cb->($api_res) if $cb;

            $blocking_cv->send($api_res) if $blocking_cv;

            return;
        }
    );

    return $blocking_cv ? $blocking_cv->recv : ();
}

# https://developer.atlassian.com/bitbucket/api/2/reference/resource/repositories/%7Busername%7D/%7Brepo_slug%7D#post
sub create_repo ( $self, $repo_id, @args ) {
    my $blocking_cv = defined wantarray ? AE::cv : undef;

    my $cb = is_plain_coderef $args[-1] ? pop @args : undef;

    my %args = (

        # common atts
        description => undef,
        has_issues  => 1,
        has_wiki    => 1,
        is_private  => 0,

        # bitbucket attrs
        scm         => $SCM_TYPE_HG,
        fork_police => 'allow_forks',    # allow_forks, no_public_forks, no_forks
        language    => 'perl',
        @args,
    );

    return $self->_req2( 'post', "/repositories/$repo_id", \%args, $cb );
}

# https://developer.atlassian.com/bitbucket/api/2/reference/resource/repositories/%7Busername%7D/%7Brepo_slug%7D#delete
sub delete_repo ( $self, $repo_id, $cb = undef ) {
    return $self->_req2( 'delete', "/repositories/$repo_id", undef, $cb );
}

# VERSIONS
# https://developer.atlassian.com/bitbucket/api/2/reference/resource/repositories/%7Busername%7D/%7Brepo_slug%7D/versions
sub get_versions ( $self, $repo_id, $cb = undef ) {
    my $blocking_cv = defined wantarray ? AE::cv : undef;

    my $versions;

    my $get = sub ($page) {
        my $sub = __SUB__;

        $self->_req2(
            'get',
            "/repositories/$repo_id/versions?page=$page&pagelen=100",
            undef,
            sub ($res) {
                if ($res) {
                    for my $ver ( $res->{data}->{values}->@* ) {
                        $versions->{ $ver->{name} } = $ver->{links}->{self}->{href};
                    }

                    if ( $res->{data}->{next} ) {
                        $sub->( ++$page );
                    }
                    else {
                        my $api_res = result 200, $versions;

                        $cb->($api_res) if $cb;

                        $blocking_cv->($api_res) if $blocking_cv;
                    }
                }
                else {
                    $cb->($res) if $cb;

                    $blocking_cv->($res) if $blocking_cv;
                }

                return;
            }
        );

        return;
    };

    $get->(1);

    return $blocking_cv ? $blocking_cv->recv : ();
}

# https://confluence.atlassian.com/bitbucket/issues-resource-296095191.html#issuesResource-POSTanewversion
sub create_version ( $self, $repo_id, $ver, $cb = undef ) {
    return $self->_req1(
        'post',
        "/repositories/$repo_id/issues/versions",
        { name => version->parse($ver)->normal },
        sub ($res) {
            if ( !$res && $res->{data}->$* =~ /already exists/sm ) {
                $res->set_status(200);
            }

            $cb->($res) if $cb;

            return;
        }
    );
}

# MILESTONES
# https://developer.atlassian.com/bitbucket/api/2/reference/resource/repositories/%7Busername%7D/%7Brepo_slug%7D/milestones
sub get_milestones ( $self, $repo_id, $cb = undef ) {
    my $blocking_cv = defined wantarray ? AE::cv : undef;

    my $versions;

    my $get = sub ($page) {
        my $sub = __SUB__;

        $self->_req2(
            'get',
            "/repositories/$repo_id/milestones?page=$page&pagelen=100",
            undef,
            sub ($res) {
                if ($res) {
                    for my $ver ( $res->{data}->{values}->@* ) {
                        $versions->{ $ver->{name} } = $ver->{links}->{self}->{href};
                    }

                    if ( $res->{data}->{next} ) {
                        $sub->( ++$page );
                    }
                    else {
                        my $api_res = result 200, $versions;

                        $cb->($api_res) if $cb;

                        $blocking_cv->($api_res) if $blocking_cv;
                    }
                }
                else {
                    $cb->($res) if $cb;

                    $blocking_cv->($res) if $blocking_cv;
                }

                return;
            }
        );

        return;
    };

    $get->(1);

    return $blocking_cv ? $blocking_cv->recv : ();
}

# https://confluence.atlassian.com/bitbucket/issues-resource-296095191.html#issuesResource-POSTanewmilestone
sub create_milestone ( $self, $repo_id, $ver, $cb = undef ) {
    return $self->_req1(
        'post',
        "/repositories/$repo_id/issues/milestones",
        { name => version->parse($ver)->normal },
        sub ($res) {
            if ( !$res && $res->{data}->$* =~ /already exists/sm ) {
                $res->set_status(200);
            }

            $cb->($res) if $cb;

            return;
        }
    );
}

# https://confluence.atlassian.com/bitbucket/issues-resource-296095191.html#issuesResource-GETalistofissuesinarepository%27stracker
sub get_issues ( $self, $repo_id, @args ) {
    my $blocking_cv = defined wantarray ? AE::cv : undef;

    my $cb = is_plain_coderef $args[-1] ? pop @args : undef;

    my %args = (
        sort   => 'priority',    # priority, kind, version, component, milestone
        status => undef,
        start  => 0,
        limit  => 50,            # 50 - max.
        @args,
    );

    # remove undefined args
    for ( keys %args ) { delete $args{$_} if !defined $args{$_} }

    my $issues;

    my $get = sub ($page) {
        my $sub = __SUB__;

        $self->_req1(
            'get',
            "/repositories/$repo_id/issues?" . P->data->to_uri( \%args ),
            undef,
            sub ($res) {
                if ($res) {
                    for my $issue ( $res->{data}->{issues}->@* ) {
                        $issues->{ $issue->{local_id} } = $issue;
                    }

                    my $api_res = result 200, data => $issues, total => $res->{data}->{count};

                    $cb->($api_res) if $cb;

                    $blocking_cv->($api_res) if $blocking_cv;
                }
                else {
                    $cb->($res) if $cb;

                    $blocking_cv->($res) if $blocking_cv;
                }

                return;
            }
        );

        return;
    };

    $get->(1);

    return $blocking_cv ? $blocking_cv->recv : ();
}

# https://confluence.atlassian.com/bitbucket/issues-resource-296095191.html#issuesResource-GETanindividualissue
sub get_issue ( $self, $repo_id, $issue_id, $cb = undef ) {
    return $self->_req1( 'get', "/repositories/$repo_id/issues/$issue_id", undef, $cb );
}

# https://confluence.atlassian.com/bitbucket/issues-resource-296095191.html#issuesResource-Updateanexistingissue
sub update_issue ( $self, $repo_id, $issue_id, $data, $cb = undef ) {
    return $self->_req1( 'put', "/repositories/$repo_id/issues/$issue_id", $data, $cb );
}

1;
## -----SOURCE FILTER LOG BEGIN-----
##
## PerlCritic profile "pcore-script" policy violations:
## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
## | Sev. | Lines                | Policy                                                                                                         |
## |======+======================+================================================================================================================|
## |    3 | 310, 315             | Subroutines::ProhibitManyArgs - Too many arguments                                                             |
## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
##
## -----SOURCE FILTER LOG END-----
__END__
=pod

=encoding utf8

=head1 NAME

Pcore::API::BitBucket

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 ATTRIBUTES

=head1 METHODS

=head1 SEE ALSO

=cut
